@@ -13,6 +13,7 @@ class BaseModel(nn.Module):
         # 这里主要是为了外面调用用到
         self.global_step, self.local_step, self.total_steps, self.epoch, self.steps_per_epoch, self.train_dataloader = 0, 0, 0, 0, None, None
         self.resume_step, self.resume_epoch = 0, 0
+        self.retain_graph = False  # loss.backward()是否保留计算图
         self.callbacks = []
         # 传入Module实例方式
         if module is not None:
@@ -80,7 +81,7 @@ class BaseModel(nn.Module):
             elif isfunction(metric):
                 self.metrics.update({metric: metric})
             else:
-                raise ValueError('Args metrics only support "String, Dict, Callback, List[String, Dict, Callback]" format')
+                raise ValueError('Args metrics only support `String, Dict, Callback, List[String, Dict, Callback]` format')
         self.stateful_metrics = stateful_metrics
 
         # 梯度累积
@@ -119,15 +120,23 @@ class BaseModel(nn.Module):
             loss = loss_detail[0]
             loss_detail = {f'loss{i}':v for i, v in enumerate(loss_detail[1:], start=1)}
         else:
-            raise ValueError('Return loss only support Tensor/dict/tuple/list format')
+            raise ValueError('Return loss only support `Tensor/dict/tuple/list` format')
 
         # 梯度累积
         loss = loss / self.grad_accumulation_steps if self.grad_accumulation_steps > 1 else loss
+
+        # backward
+        self.scale_before_step = 0
+        if self.use_amp:  # 混合精度
+            self.scale_before_step = self.scaler.get_scale()
+            self.scaler.scale(loss).backward(retain_graph=self.retain_graph)
+        else:
+            loss.backward(retain_graph=self.retain_graph)
         return output, loss, loss_detail
 
     def fit(self, train_dataloader, steps_per_epoch=None, epochs=1, callbacks=None, verbose=1):
         if not hasattr(train_dataloader, '__len__'):
-            assert steps_per_epoch is not None, 'Either train_dataloader has attr "__len__" or steps_per_epoch is not None'
+            assert steps_per_epoch is not None, 'Either train_dataloader has attr `__len__` or steps_per_epoch is not None'
 
         self.steps_per_epoch = len(train_dataloader) if steps_per_epoch is None else steps_per_epoch
         self.total_steps = self.steps_per_epoch * epochs
@@ -140,11 +149,12 @@ class BaseModel(nn.Module):
         if not isinstance(callbacks, (list, tuple)):
             callbacks = [callbacks]
         for callback in callbacks:
-            assert isinstance(callback, Callback), "Args 'callbacks' only support Callback() inputs"
+            assert isinstance(callback, Callback), "Args `callbacks` only support Callback() inputs"
         progbarlogger = ProgbarLogger(stateful_metrics=self.stateful_metrics)  # 进度条
         history = History()
-        self.callbacks = CallbackList([BaseLogger(self.stateful_metrics), progbarlogger] + callbacks + [history])
-        callback_model = self.module if hasattr(self, 'module') else self
+        master_rank = self.master_rank if hasattr(self, 'master_rank') else None
+        self.callbacks = CallbackList([BaseLogger(self.stateful_metrics), progbarlogger] + callbacks + [history], master_rank=master_rank)
+        callback_model = self
         self.callbacks.set_model(callback_model)
         self.callbacks.set_params({
             'epochs': epochs,
@@ -175,31 +185,29 @@ class BaseModel(nn.Module):
                 try:
                     batch = next(train_dataloader_iter)
                 except StopIteration:
-                    self.callbacks.on_dataloader_end()  # 适用于数据量较大时，动态读取文件并重新生成dataloader的情况，如预训练
+                    self.callbacks.on_dataloader_end()  # 适用于数据量较大时，动态读取文件并重新生成self.train_dataloader的情况，如预训练
                     train_dataloader_iter = iter(self.train_dataloader)  # shuffle=True时候，其实顺序也重新生成了
                     self.bti = 0
                     batch = next(train_dataloader_iter)
-                train_X, train_y = batch
+                self.train_X, self.train_y = batch
 
                 # 从train_X中取batch_size，最多允许嵌套两层，即encoder和decoder的((token_ids1, mask1), (token_ids2, mask2))
-                if isinstance(train_X, (list, tuple)) and isinstance(train_X[0], (list, tuple)):
-                    btz = train_X[0][0].size(0)
-                elif isinstance(train_X, (list, tuple)) and (not isinstance(train_X[0], (list, tuple))):
-                    btz = train_X[0].size(0)
-                elif isinstance(train_X, torch.Tensor):
-                    btz = train_X.size(0)
+                if isinstance(self.train_X, (list, tuple)) and isinstance(self.train_X[0], (list, tuple)):
+                    btz = self.train_X[0][0].size(0)
+                elif isinstance(self.train_X, (list, tuple)) and (not isinstance(self.train_X[0], (list, tuple))):
+                    btz = self.train_X[0].size(0)
+                elif isinstance(self.train_X, torch.Tensor):
+                    btz = self.train_X.size(0)
                 else:
-                    raise ValueError('Input only support [list, tuple, tensor]')
+                    raise ValueError('Input only support `[list, tuple, tensor]`')
                 logs = {'size': btz}
                 self.callbacks.on_batch_begin(self.global_step, self.local_step, logs)
 
                 self.train()  # 设置为train模式
                 # 入参个数判断，如果入参>=3表示是多个入参，如果=2则表示是一个入参
-                output, loss, loss_detail = self.train_step(train_X, train_y)
-                
-                # 主要是方便bert4torch中的对抗训练
-                scale_before_step, loss, loss_detail = self.after_train_step(train_X, train_y, output, loss, loss_detail)
-                
+                self.output, self.loss, self.loss_detail = self.train_step(self.train_X, self.train_y)
+                self.callbacks.on_train_step_end()
+                                
                 # 参数更新, 真实的参数更新次数要除以grad_accumulation_steps，注意调整总的训练步数
                 if (self.global_step+1) % self.grad_accumulation_steps == 0:
                     skip_scheduler = False
@@ -210,7 +218,7 @@ class BaseModel(nn.Module):
                             torch.nn.utils.clip_grad_norm_(self.parameters(), self.clip_grad_norm)
                         self.scaler.step(self.optimizer)
                         self.scaler.update()
-                        skip_scheduler = self.scaler.get_scale() != scale_before_step
+                        skip_scheduler = self.scaler.get_scale() != self.scale_before_step
                     else:
                         if self.clip_grad_norm is not None:  # 梯度裁剪
                             torch.nn.utils.clip_grad_norm_(self.parameters(), self.clip_grad_norm)
@@ -225,15 +233,15 @@ class BaseModel(nn.Module):
                             self.scheduler.step()
 
                 # 添加loss至log打印
-                logs.update({'loss': loss.item()})
-                logs_loss_detail = {k: v.item() if isinstance(v, torch.Tensor) else v for k, v in loss_detail.items()}
+                logs.update({'loss': self.loss.item()})
+                logs_loss_detail = {k: v.item() if isinstance(v, torch.Tensor) else v for k, v in self.loss_detail.items()}
                 logs.update(logs_loss_detail)
                 if self.global_step == resume_step:
                     progbarlogger.add_metrics(list(logs_loss_detail.keys()), add_position=1)
                     
                 # 添加metrics至log打印
                 for metric, func in self.metrics.items():
-                    perf = metric_mapping(metric, func, output, train_y)  # 内置的一些accuracy指标
+                    perf = metric_mapping(metric, func, self.output, self.train_y)  # 内置的一些accuracy指标
                     if perf is not None:
                         if isfunction(metric):  # 直接传入回调函数(无key)
                             if self.global_step == resume_step:
@@ -262,15 +270,19 @@ class BaseModel(nn.Module):
             return output[return_all]
         else:
             raise ValueError('Return format error')
-    
-    def after_train_step(self, train_X, train_y, output, loss, loss_detail):
-        scale_before_step = 0
-        if self.use_amp:  # 混合精度
-            scale_before_step = self.scaler.get_scale()
-            self.scaler.scale(loss).backward()
-        else:
-            loss.backward()
-        return scale_before_step, loss, loss_detail
+
+    def load_weights(self, load_path, strict=True):
+        '''加载模型权重
+           save_path: 权重加载路径
+        '''
+        state_dict = torch.load(load_path, map_location='cpu')
+        self.load_state_dict(state_dict, strict=strict)
+
+    def save_weights(self, save_path):
+        '''保存模型权重
+           save_path: 权重保存路径
+        '''
+        torch.save(self.state_dict(), save_path)
 
 
 class BaseModelDP(nn.DataParallel, BaseModel):
